/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.readGraph.getSingleObject;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.procedure.adapter.ListenerAdapter;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;
import org.simantics.db.request.Read;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;
import org.simantics.utils.DataContainer;

/**
 * Tests cases where a non-functional relation uses
 * {@link ReadGraph#getSingleObject(org.simantics.db.Resource, org.simantics.db.Resource)}
 * with > 1 objects.
 * 
 * <p>
 * Should throw {@link NoSingleResultException}, at the time of writing this
 * test, AsyncBarrierImpl refcounting is trashed and DB client is stuck.
 */
public class GetSingleObjectTest3_bak extends ExistingDatabaseTest {

	@Test
    public void testSingleObject() throws Exception {
        Session session = getSession();

        final DataContainer<Resource> nonfunctionalRelation = new DataContainer<Resource>();
        final DataContainer<Resource> subject = new DataContainer<Resource>();
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    // Use any non-functional relation
                    Layer0 b = Layer0.getInstance(graph);

                    Resource rel = graph.newResource();
                    graph.claim(rel, b.SubrelationOf, b.IsRelatedTo);

                    Resource rel2 = graph.newResource();
                    graph.claim(rel2, b.SubrelationOf, rel);

                    Resource s = graph.newResource();
                    graph.claim(s, b.InstanceOf, b.Entity);

                    Resource o1 = graph.newResource();
//                    Resource o2 = graph.newResource();

                    graph.claim(s, rel, o1);
//                    graph.claim(s, rel, o2);

                    nonfunctionalRelation.set(rel);
                    subject.set(s);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        try {
            session.syncRequest(new Read<Resource>() {
                @Override
                public Resource perform(ReadGraph graph) throws DatabaseException {
                    return graph.getSingleObject(subject.get(), nonfunctionalRelation.get());
                }
            }, new ListenerAdapter<Resource>() {

				@Override
				public boolean isDisposed() {
					return false;
				}
            	
            });
        } catch (NoSingleResultException e) {
            System.out.println("Read transaction threw an expected exception " + e);
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }
        
        if (DEBUG)
            System.out.println("################################################");
        
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    graph.claim(subject.get(), nonfunctionalRelation.get(), nonfunctionalRelation.get());
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }
        
    }

}
