/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.readGraph.getSingleStatement;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;
import org.simantics.db.testing.base.WriteReadTest;

/**
 * Tests cases where a non-functional relation uses
 * {@link ReadGraph#getSingleObject(org.simantics.db.Resource, org.simantics.db.Resource)}
 * with > 1 objects.
 * 
 * <p>
 * Should throw {@link NoSingleResultException}, at the time of writing this
 * test, AsyncBarrierImpl refcounting is trashed and DB client is stuck.
 */
public class GetSingleStatementTest23 extends WriteReadTest {
	private Resource subject1;
	private Resource relation1;
	private Resource object1;

	@Override
	protected void write(WriteGraph graph) throws DatabaseException {

		relation1 = graph.newResource();
		graph.claim(relation1, L0.SubrelationOf, L0.IsRelatedTo);

		//Resource rel2 = graph.newResource();
		//graph.claim(rel2, L0.SubrelationOf, relation1);

		subject1 = graph.newResource();
		graph.claim(subject1, L0.InstanceOf, L0.Entity);

		object1 = graph.newResource();
		Resource o2 = graph.newResource();

		graph.claim(subject1, relation1, object1);
		graph.claim(subject1, relation1, o2);
	}

	@Override
	protected void read(ReadGraph graph) throws DatabaseException {
		try {

			assertNotNull("Null not allowed", graph.getSingleStatement(subject1, relation1));
			//assertTrue("Wrong object was returned", (object1.equals(object)));

		} catch (NoSingleResultException e) {
			//System.out.println("Read transaction threw an expected exception "
			//		+ e);
			return;
		} catch (Throwable e) {
			fail("Read transaction threw an unexpected exception " + e);
		}
		fail("Should throw");

		/*
		 * try {
		 * 
		 * graph.claim(subject1, relation1, relation1);
		 * 
		 * } catch (Throwable e) {
		 * fail("Write transaction threw an unknown exception " + e); }
		 */
	}

}
