/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.external;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Session;
import org.simantics.db.procedure.Listener;
import org.simantics.db.request.ExternalRead;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.base.ExistingDatabaseTest;

public class ExternalRequestTest extends ExistingDatabaseTest {
	
	private Error exception;
	
	int counter = 0;
	
	@Test
	public void test() throws Exception {
		
		Session session = getSession();
		
		final ScheduledExecutorService exec = Executors.newScheduledThreadPool(1);

		final Semaphore s = new Semaphore(-9);
		
		session.syncRequest(new ExternalRead<String>() {

            @Override
            public void register(ReadGraph graph, final Listener<String> procedure) {

                procedure.execute("String" + (counter++));

                for(int i=1;i<10;i++) {
                    exec.schedule(new Runnable() {
    
                        @Override
                        public void run() {
                            procedure.execute("String" + (counter++));
                        }
                        
                    }, i*500, TimeUnit.MILLISECONDS);
                }
                
            }
            
            @Override
            public void unregistered() {
            }
			
		}, new Listener<String>() {

            @Override
            public void exception(Throwable t) {
            }

            @Override
            public void execute(String result) {
                s.release();
            }

            @Override
            public boolean isDisposed() {
                return false;
            }
		    
		});
		
		s.acquire();
		
		if(exception != null) {
			fail("Write transaction threw and exception (" + exception.getMessage() + ") which was not passed through ");
		}
		
	}

}
