/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.misc;

import java.util.Set;

import org.junit.Test;
import org.simantics.db.AsyncReadGraph;
import org.simantics.db.Session;
import org.simantics.db.impl.query.CacheEntry;
import org.simantics.db.impl.service.QueryDebug;
import org.simantics.db.procedure.AsyncListener;
import org.simantics.db.procedure.AsyncProcedure;
import org.simantics.db.request.AsyncRead;
import org.simantics.db.testing.base.ExistingDatabaseTest;

public class RequestParentTest extends ExistingDatabaseTest {
	
	@Test
	public void test() throws Exception {
		
		Session session = getSession();
		
		QueryDebug debug = session.getService(QueryDebug.class);
		
        class Request implements AsyncRead<Object> {

            @Override
            public void perform(AsyncReadGraph graph, AsyncProcedure<Object> procedure) {
                procedure.execute(graph, new Object());
            }

            @Override
		    public int threadHash() {
		    	return hashCode();
		    }

            @Override
            public int getFlags() {
                return 0;
            }
 
        }
		
        final Request request1 = new Request();
        final Request request2 = new Request();
        
        session.syncRequest(request1, new AsyncListener<Object>() {

            @Override
            public void exception(AsyncReadGraph graph, Throwable t) {
            }

            @Override
            public void execute(AsyncReadGraph graph, Object result) {
                graph.asyncRequest(request2);
            }

            @Override
            public boolean isDisposed() {
                return false;
            }

        });
		
		Set<CacheEntry> parents = debug.getParents(request2);
		for(CacheEntry parent : parents) {
		    if(parent.getOriginalRequest().equals(request1)) fail("Request1 should not be parent of Request2");
		}
		if(!parents.isEmpty())fail("Request2 should not have parents");
		
	}

}
