/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.thread;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Session;
import org.simantics.db.common.procedure.adapter.ListenerAdapter;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.request.Read;
import org.simantics.db.service.QueryControl;
import org.simantics.db.testing.base.ExistingDatabaseTest;

/*
 * This test ensures that synchronous requests are processed in same thread
 * 
 */
public class ThreadingTest1 extends ExistingDatabaseTest {
	
	@Test
    public void test() throws Exception {
		
		Session session = getSession();
		
		QueryControl control = session.getService(QueryControl.class);
		if(control.getAmountOfQueryThreads() == 1) return;
		
		int result = session.syncRequest(new Read<Integer>() {

            @Override
            public Integer perform(ReadGraph graph) throws DatabaseException {
            	
            	final int thread = graph.thread();
                
            	return graph.syncRequest(new Read<Integer>() {

					@Override
					public Integer perform(ReadGraph graph) throws DatabaseException {
						
						assert(thread == graph.thread());
						
						return 1;
						
					}
					
            	});
                
            }
            
            @Override
            public int hashCode() {
            	return 0;
            }
		    
		}, new ListenerAdapter<Integer>());
		
		assert(result == 1);
		
	}

}
