/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.exception.NoSingleResultException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;
import org.simantics.utils.DataContainer;

/**
 * Tests cases where a non-functional relation uses
 * {@link ReadGraph#getSingleObject(org.simantics.db.Resource, org.simantics.db.Resource)}
 * with > 1 objects.
 * 
 * <p>
 * Should throw {@link NoSingleResultException}, at the time of writing this
 * test, AsyncBarrierImpl refcounting is trashed and DB client is stuck.
 */
public class RedoTest1 extends ExistingDatabaseTest {

	@Test
    public void test() throws Exception {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        final DataContainer<Resource> subject = new DataContainer<Resource>();
        
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    // Use any non-functional relation
                    Layer0 b = Layer0.getInstance(graph);
                    Resource s = graph.newResource();
                    graph.claim(s, b.InstanceOf, b.Entity);
                    subject.set(s);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    assert(graph.hasStatement(subject.get()));
                }
            });
        } catch (Throwable e) {
            fail("Could not validate statement. Read transaction threw an unexpected exception " + e);
        }

        support.undo(session, 1);
        
        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                	assert(!graph.hasStatement(subject.get()));
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }
        
        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    assert(!graph.hasStatement(subject.get()));
                }
            });
        } catch (Throwable e) {
            fail("Could not validate statement's undo. Read transaction threw an unexpected exception " + e);
        }

        support.redo(session, 1);

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                	assert(graph.hasStatement(subject.get()));
                }
            });
        } catch (Throwable e) {
            fail("Could not validate statement's redo. Read transaction threw an unexpected exception " + e);
        }
        
    }

}
