/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.write.deny;

import java.util.UUID;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.VirtualGraph;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.common.request.WriteResultRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.VirtualGraphSupport;
import org.simantics.db.testing.base.WriteReadTest;

public class DenyTest1 extends WriteReadTest {
	
	Resource other;
	
	@Override
	protected void write(WriteGraph graph) throws DatabaseException {
		
		VirtualGraphSupport support = graph.getService(VirtualGraphSupport.class);
		VirtualGraph virt = support.getMemoryPersistent(UUID.randomUUID().toString());

		final Resource other2 = graph.syncRequest(new WriteResultRequest<Resource>(virt) {

			@Override
			public Resource perform(WriteGraph graph) throws DatabaseException {
				Resource other = graph.newResource();
				graph.claim(written, L0.ConsistsOf, L0.PartOf, other);
				return other;
			}
			
		});

		Resource single = graph.getSingleObject(written, L0.ConsistsOf);
		assertNotNull(single);
		assertTrue(single.equals(other2));
		
		other = graph.newResource();
		graph.claim(written, L0.ConsistsOf, L0.PartOf, other);

		graph.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph graph) throws DatabaseException {
				graph.deny(written, L0.ConsistsOf, L0.PartOf, other2);
			}
			
		});
		
	}
	
	@Override
	protected void read(ReadGraph graph) throws DatabaseException {
		
		Resource single = graph.getPossibleObject(written, L0.ConsistsOf);
		assertNotNull(single);
		assertTrue(single.equals(other));
		
	}
	
}